# Data Frames sind eine grundlegende und oft genutzte Datenstruktur in R.
# Man kann sich einen Data Frame als eine Tabelle mit benannten Spalten und Zeilen vorstellen.
# Demnach ist ein Data Frame also zweidimensional.

# Große Besonderheit: Data Frames erlauben verschiedene Datentypen (Von Spalte zu Spalte)
# So lassen sich Daten gut strukturieren und ganze Datensätze darstellen.

## Erzeugung:
# Zuerst werden Eingabeargumente benötigt. In diesem Beispiel werden dafür drei Vektoren benutzt.
> integer = c(1L,2L,3L,4L)
> character = c("a","b","c","d")
> logical = c(TRUE,FALSE,TRUE,FALSE)

# Dann erzeugen wir den Data Frame mit den entsprechenden Eingabeargumenten.
> df = data.frame(integer,character,logical)
> df
  integer character logical
1       1         a    TRUE
2       2         b   FALSE
3       3         c    TRUE
4       4         d   FALSE

## Benennung
# Manchmal kann es sinnvoll sein, die Spalten - oder Zeilenbeschriftungen anzupassen.

# Spaltennamen:
> colnames(df)
[1] "integer"   "character" "logical"  

# Modifizieren der Spaltennamen:
> colnames(df) = c("ich","bin","test")
> colnames(df)
[1] "ich"  "bin"  "test"

# Zeilennamen:
> rownames(df)
[1] "1" "2" "3" "4"

# Modifizieren der Zeilennamen:
> rownames(df) = c(5,6,7,8)
> rownames(df)
[1] "5" "6" "7" "8"

# Nun sieht der Data Frame nun aus:
> df
  ich bin  test
5   1   a  TRUE
6   2   b FALSE
7   3   c  TRUE
8   4   d FALSE

## Kombinieren
# Kombinieren von Spalten: 
> df = cbind(df, test2 = c(10,11,12,13))
> df
  ich bin  test test2
5   1   a  TRUE    10
6   2   b FALSE    11
7   3   c  TRUE    12
8   4   d FALSE    13

# Kombinieren von Zeilen:
# Dies ist etwas kniffliger, da eine Zeile ja ggf. mehrere Datentypen beinhaltet. Am besten löst man dies durch Erzeugung eines neuen Data Frames.
# Dort gibt man nun korrekt die Spaltennamen des Data Frames an, mit dem man mergen möchte. Dann funktioniert auch das Hinzufügen von Zeilen.
> neu = data.frame(ich = 5, bin ="e",test =TRUE,test2 = 14)
> df = rbind(df, neu)
> df
     ich  bin  test test2
5      1    a  TRUE    10
6      2    b FALSE    11
7      3    c  TRUE    12
8      4    d FALSE    13
5      5    e  TRUE    14
# Hier müsste man jetzt noch ggf. den Zeilennamen der fünften Zeile anpassen. 

## Adressierung
# (Wir erstellen an dieser Stelle zuerst einen neuen Data Frame.)
# Mit dem Argument "stringsAsFactors = FALSE" stellen wir sicher, dass characters eben nicht als Faktor abgespeichert werden.
# Das macht es uns möglich, später Zeilen oder Spalten zu modifizieren, und zwar auch mit characters, die nicht bei der Erzeugung des Data Frames bekannt waren.
# 
> df = data.frame(integer = c(7,8,9), character = c("a","b","c"), logical = c(TRUE,FALSE,TRUE), complex = c(2+2i, 4-3i, 1+1i), stringsAsFactors = FALSE)
> df
  integer character logical complex
1       7         a    TRUE    2+2i
2       8         b   FALSE    4-3i
3       9         c    TRUE    1+1i

# Adressieren und Modifizieren von Zeilen:
> df[1,]
  integer character logical complex
1       7         a    TRUE    2+2i
# Auf diese Weise können wir ganze Zeilen ansprechen. In diesem Falle die erste Zeile.
# Nun können wir ganz einfach auch den Inhalt ändern: 
> df[1,] = list(10,"m",FALSE,3-3i)
> df
  integer character logical complex
1      10         m   FALSE    3-3i
2       8         b   FALSE    4-3i
3       9         c    TRUE    1+1i

# Adressieren und Modifizieren von Spalten:
> df[,1]
[1] 10  8  9
# Auf diese Weise können wir ganze Spalten ansprechen. In diesem Falle die erste Spalte.
# Nun können wir ganz einfach auch den Inhalt ändern:
> df[,1] = c(11,12,13)
> df
  integer character logical complex
1      11         m   FALSE    3-3i
2      12         b   FALSE    4-3i
3      13         c    TRUE    1+1i








